/*
 * Copyright (c) 1999-2021 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_COLOR_HPP
#define PW_CORE_COLOR_HPP

#include <pw/core/globals.hpp>
#include <pw/core/vector.hpp>


namespace pw {

struct color {

    vector4f rgba{vector4f::all(1)};

    static constexpr auto from_rgb8888(uint8_t r8, uint8_t g8, uint8_t b8,
                                       uint8_t a8) -> color {
        return {static_cast<float>(r8) / std::numeric_limits<uint8_t>::max(),
                static_cast<float>(g8) / std::numeric_limits<uint8_t>::max(),
                static_cast<float>(b8) / std::numeric_limits<uint8_t>::max(),
                static_cast<float>(a8) / std::numeric_limits<uint8_t>::max()};
    }

    static constexpr auto from_rgb8888(uint32_t v) -> color {
        return from_rgb8888((v & 0xff000000) >> 24, (v & 0x00ff0000) >> 16,
                            (v & 0x0000ff00) >> 8, (v & 0x000000ff));
    }

    uint32_t to_rgb8888() const {
        return 0;
    }
};

} // namespace pw

#endif
